package com.qmth.themis.business.listener;

import com.alibaba.fastjson.JSONObject;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBSession;
import com.qmth.themis.business.service.TBSessionService;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import org.apache.rocketmq.client.consumer.DefaultMQPushConsumer;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.client.consumer.listener.MessageListenerConcurrently;
import org.apache.rocketmq.common.message.Message;
import org.apache.rocketmq.common.message.MessageExt;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.annotation.SelectorType;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.apache.rocketmq.spring.core.RocketMQPushConsumerLifecycleListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @Description: 普通消息监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/28
 */
@Service
public class RocketConsumer {

    private final static Logger log = LoggerFactory.getLogger(RocketConsumer.class);

    @Resource
    TBSessionService tbSessionService;

    @Service
    @RocketMQMessageListener(consumerGroup = "themis-group", topic = "themis-session-topic", selectorType = SelectorType.TAG, selectorExpression = "web||wxapp||pc")
    public class sessionConsumer implements RocketMQListener<Message>, RocketMQPushConsumerLifecycleListener {

        @Override
        public void onMessage(Message message) {
            //实现RocketMQPushConsumerLifecycleListener监听器之后，此方法不调用
            log.info("实现RocketMQPushConsumerLifecycleListener监听器之后，此方法不调用");
        }

        @Override
        public void prepareStart(DefaultMQPushConsumer defaultMQPushConsumer) {
            defaultMQPushConsumer.setConsumeMessageBatchMaxSize(SystemConstant.CONSUME_MESSAGE_BATCH_MAX_SIZE);//每次拉取10条
            defaultMQPushConsumer.setMaxReconsumeTimes(SystemConstant.MAXRECONSUMETIMES);//最大重试次数
            defaultMQPushConsumer.registerMessageListener(new MessageListenerConcurrently() {
                @Override
                public ConsumeConcurrentlyStatus consumeMessage(List<MessageExt> msgs, ConsumeConcurrentlyContext context) {
                    for (MessageExt messageExt : msgs) {
                        log.info("重试次数:{}", messageExt.getReconsumeTimes());
                        JSONObject jsonObject = JSONObject.parseObject(new String(messageExt.getBody(), Constants.CHARSET));
                        log.info("接受到的消息:{}", jsonObject.toJSONString());
                        Map map = (Map) jsonObject.get(SystemConstant.PROPERTIES);
                        TBSession tbSession = JSONObject.toJavaObject(JSONObject.parseObject(String.valueOf(map.get(String.valueOf(map.get("TAGS"))))), TBSession.class);
                        tbSessionService.saveSessionInfo(tbSession);
                        tbSession.setAck(SystemConstant.STANDARD_ACK_TYPE);
                        RedisUtil.setUserSession(tbSession.getId(), tbSession);
                        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;
                    }
                    return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;//成功
//                    return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
                }
            });
        }
    }
}
